#!/bin/sh 
# This script is to manage gps data from android ShareGPS 
# It simply redirects tcp data to fifo (with ncat) 
# Then starts gpsd daemon using that fifo as gps device. 
# Requires: 
# adb from android sdk 
# gpsd - linux gps daemon 
# ncat 
# 
# Run that script manually or better via udev rule 
#################### Some definitions to edit ################# 
# adb binary location 
ADB=adb
LOG="YES"
LOGFILE=/tmp/adb_gps.log
# Other stuff
NCAT=ncat
NETSTAT=netstat
ADBGPSpid="/tmp/adb_gps_usb.pid"
# Select local port to bind with adb 
LOCAL_PORT=50000 
# gpsd binary location 
GPSD=/usr/sbin/gpsd 
GPSDPID=/tmp/gpsd.pid 
# That's command to stop not needed gpsd service 
# If necessarily replace with your system command 
GPSDSTOP="/usr/sbin/service gpsd stop" 
ADBFIFO=/tmp/adb_gps.fifo
################## Definitins end ############################# 
[ "$LOG" = "YES" ] && LOG="$LOGFILE" && echo Loging enabled >> $LOGFILE || LOG=/dev/null

stop_active_adb(){ 
# to be sure any other adb proccess doesn't mess our connection 
[ -n "`ps aux | grep adb`" ] && $ADB kill-server 2>>$LOGFILE
# and again forced
#killall adb
}
 
# Remove possibly stale fifo, then create new 
check_pipe(){ 
rm -f $ADBFIFO 
mkfifo $ADBFIFO 
}
# Enable adb tcp port forwarding 
connect_adb(){ 
$ADB forward tcp:$LOCAL_PORT tcp:50000 2>>$LOGFILE
} 
stop(){
kill -3 `cat $GPSDPID` 2>>$LOGFILE
# very dirty - to be sure :> 
#killall -9 gpsd 2>>$LOGFILE
#stop_active_adb
rm -f $ADBFIFO
# Close possile stale script may be even self
for i in `cat $ADBGPSpid`;do
kill -3 $i 2>>$LOGFILE
# Next force cleanup
kill -9 $i 2>>$LOGFILE
done
rm -f $ADBGPSpid
exit 0 
} 
redirect_data(){ 
# Command to check if there is connection to ShareGPS service 
ADBSTATUS=`$NETSTAT -nlp | grep $LOCAL_PORT | grep adb`
echo "Start ADBSTATUS=$ADBSTATUS" 2>>$LOGFILE
# Run until there is active connection on selected port 
while [ -n "$ADBSTATUS" ]; do 
# Redirect tcp data to fifo
#echo "Start ncat"
$NCAT --recv-only localhost $LOCAL_PORT > $ADBFIFO 
# When above redirection fail but adb is still active 
# try to redirect data again 
ADBSTATUS=`$NETSTAT -nlp | grep $LOCAL_PORT | grep adb`
echo "ADBSTATUS=$ADBSTATUS `date`" 2>>$LOGFILE
done
# When adb connection is lost stop and clean 
stop 
} 
start(){
# Write script pid
ps aux | grep adb_gps_usb.sh | grep -v grep | awk '{print $2" "}' >> $ADBGPSpid
check_pipe
stop_active_adb
sleep 1
connect_adb
# Be sure, system gpsd service is not runnig 
$GPSDSTOP 
# Start gpsd 
$GPSD -b -P $GPSDPID $ADBFIFO 
redirect_data 
} 
case $1 in
	start)
        echo "Start at `date`
############################" >>$LOGFILE
	start
	;;
	stop) 
	echo "Stop at `date`
###########################" >>$LOGFILE
	stop
	;;
	restart)
	start
	stop
	;;
	*)
	echo "Usage start|stop|restart"
esac

